/**
 * New Relic Instrumentation Service
 *
 * A TypeScript service that provides a wrapper around New Relic browser APIs.
 * This service uses lazy initialization to access New Relic only when needed,
 * ensuring that it works even if the host application initializes New Relic
 * after this service is instantiated - or not at all.
 */
export class NewRelicInstrumentationService {
    /**
     * Creates an instance of NewRelicInstrumentationService.
     * @param document - The Document object, used to access the window
     */
    constructor(document) {
        this.document = document;
    }
    /**
     * Gets the New Relic instance from the window object
     * This is called lazily when needed, not at initialization time
     * @returns The New Relic instance or undefined if not available
     */
    getNewRelic() {
        try {
            const window = this.document.defaultView;
            return window && window.NREUM ? window.NREUM : undefined;
        }
        catch (error) {
            console.warn("Failed to access New Relic:", error);
            return undefined;
        }
    }
    /**
     * Check if New Relic is currently available
     * @returns boolean indicating whether New Relic is available
     */
    isAvailable() {
        return !!this.getNewRelic();
    }
    // ==============================
    // Traces and Events Methods
    // ==============================
    /**
     * Add a page action (custom event) to New Relic
     * @param eventName - Name of the event
     * @param attributes - Attributes associated with the event
     */
    addPageAction(eventName, attributes) {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.addPageAction(eventName, attributes);
        }
    }
    /**
     * Add information to an in-progress session trace
     * @param attribute - The attribute to add to the trace
     */
    addToTrace(attribute) {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.addToTrace(attribute);
        }
    }
    /**
     * Record a time point when the page is finished according to custom criteria
     */
    finished() {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.finished();
        }
    }
    /**
     * Add a custom attribute to subsequent events on the page
     * @param key - The attribute name
     * @param value - The attribute value
     */
    setCustomAttribute(key, value) {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.setCustomAttribute(key, value);
        }
    }
    /**
     * Add application version information to subsequent events
     * @param version - The application version
     */
    setApplicationVersion(version) {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.setApplicationVersion(version);
        }
    }
    /**
     * Group page views by setting a page name
     * @param name - The page name
     */
    setPageViewName(name) {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.setPageViewName(name);
        }
    }
    /**
     * Add user ID information to subsequent events on the page
     * @param userId - The user ID
     */
    setUserId(userId) {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.setUserId(userId);
        }
    }
    /**
     * Capture a single browser log event
     * @param message - The log message
     * @param level - Log level (optional)
     * @param context - Additional context (optional)
     */
    log(message, level, context) {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.log(message, level, context);
        }
    }
    /**
     * Wrap an existing logger to capture messages as log events
     * @param logger - The logger to wrap
     */
    wrapLogger(logger) {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.wrapLogger(logger);
        }
    }
    // ==============================
    // Error Reporting Methods
    // ==============================
    /**
     * Tag errors with your app's version information
     * @param releaseName - The release name/version
     */
    addRelease(releaseName) {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.addRelease(releaseName);
        }
    }
    /**
     * Capture a caught or handled error without disrupting the app's operation
     * @param error - The error object
     * @param customAttributes - Optional custom attributes
     */
    noticeError(error, customAttributes) {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.noticeError(error, customAttributes);
        }
    }
    /**
     * Set a custom error handler to ignore specific errors
     * @param errorHandler - Function that returns true if the error should be ignored
     */
    setErrorHandler(errorHandler) {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.setErrorHandler(errorHandler);
        }
    }
    // ==============================
    // SPA Tracking Methods
    // ==============================
    /**
     * Return a new API object bound to the current interaction
     * If New Relic is not available, returns a no-op implementation
     */
    interaction() {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            return newrelic.interaction();
        }
        // Return a no-op implementation to prevent errors
        return {
            actionText: () => this,
            createTracer: () => () => { },
            end: () => { },
            getContext: () => ({}),
            ignore: () => { },
            onEnd: () => this,
            save: () => this,
            setAttribute: () => this,
            setCurrentRouteName: () => this
        };
    }
    /**
     * Set the actionText of a SPA interaction
     * @param text - The text of the HTML element clicked
     */
    setInteractionActionText(text) {
        const interaction = this.interaction();
        interaction.actionText(text);
    }
    /**
     * End a SPA interaction
     */
    endInteraction() {
        const interaction = this.interaction();
        interaction.end();
    }
    /**
     * Add a custom attribute to the current interaction
     * @param key - The attribute name
     * @param value - The attribute value
     */
    setInteractionAttribute(key, value) {
        const interaction = this.interaction();
        interaction.setAttribute(key, value);
    }
    /**
     * Give SPA routes more accurate names than default names
     * @param routeName - The route name
     */
    setCurrentRouteName(routeName) {
        const interaction = this.interaction();
        interaction.setCurrentRouteName(routeName);
    }
    /**
     * Ignore the current interaction
     */
    ignoreInteraction() {
        const interaction = this.interaction();
        interaction.ignore();
    }
    /**
     * Save the current interaction
     */
    saveInteraction() {
        const interaction = this.interaction();
        interaction.save();
    }
    // ==============================
    // Agent Control Methods
    // ==============================
    /**
     * Start harvesting one or more features
     * @param features - Array of features to start
     */
    start(features) {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.start(features);
        }
    }
    /**
     * Manually start session replay collection
     */
    recordReplay() {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.recordReplay();
        }
    }
    /**
     * Manually pause session replay collection
     */
    pauseReplay() {
        const newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.pauseReplay();
        }
    }
}
/**
 * Base class for clients that need New Relic instrumentation
 *
 * This abstract class provides common New Relic initialization and tracking
 * functionality that can be reused across different client implementations.
 */
export class NewRelicEnabledClient {
    /**
     * Constructor initializes New Relic automatically
     */
    constructor() {
        this.newRelic = null;
        // Initialize New Relic on construction
        try {
            this.newRelic = new NewRelicInstrumentationService(document);
            if (!(this.newRelic && this.newRelic.isAvailable())) {
                console.warn("New Relic is initialized but not available");
                this.newRelic = null;
            }
        }
        catch (error) {
            console.warn("Failed to initialize New Relic instrumentation:", error);
            this.newRelic = null;
        }
    }
    /**
     * Track API activity or errors in New Relic
     */
    track(type, name, attributes, error) {
        if (!this.newRelic)
            return;
        const commonAttrs = Object.assign(Object.assign(Object.assign({}, attributes), { timestamp: new Date().toISOString() }), this.getTrackingBaseAttributes());
        if (type === "activity") {
            this.newRelic.addPageAction(name, commonAttrs);
        }
        else if (error) {
            this.newRelic.noticeError(error, commonAttrs);
        }
    }
    /**
     * Get network information for diagnostics
     */
    getNetworkInfo() {
        const info = {
            online: typeof navigator !== "undefined" && navigator.onLine
        };
        if (typeof navigator !== "undefined" && "connection" in navigator) {
            const connection = navigator.connection;
            if (connection) {
                info.effectiveType = connection.effectiveType || "unknown";
                info.downlink = connection.downlink || "unknown";
                info.rtt = connection.rtt || "unknown";
                info.saveData = connection.saveData || false;
            }
        }
        return info;
    }
}
